/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.builders.snapshot;

import java.util.Collections;
import java.util.List;
import java.util.Random;

import javax.annotation.Nullable;

import net.minecraft.init.Biomes;
import net.minecraft.init.Blocks;
import net.minecraft.profiler.Profiler;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.DimensionType;
import net.minecraft.world.GameType;
import net.minecraft.world.World;
import net.minecraft.world.WorldProvider;
import net.minecraft.world.WorldSettings;
import net.minecraft.world.WorldType;
import net.minecraft.world.biome.Biome;
import net.minecraft.world.biome.BiomeProvider;
import net.minecraft.world.chunk.IChunkProvider;
import net.minecraft.world.gen.layer.GenLayer;
import net.minecraft.world.storage.SaveHandlerMP;
import net.minecraft.world.storage.WorldInfo;

import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import buildcraft.api.schematics.ISchematicBlock;

@SuppressWarnings("NullableProblems")
@SideOnly(Side.CLIENT)
public class FakeWorld extends World {
    private static final Biome BIOME = Biomes.field_76772_c;
    @SuppressWarnings("WeakerAccess")
    public static final BlockPos BLUEPRINT_OFFSET = new BlockPos(0, 127, 0);

    @SuppressWarnings("WeakerAccess")
    public FakeWorld() {
        super(
            new SaveHandlerMP(),
            new WorldInfo(
                new WorldSettings(
                    0,
                    GameType.CREATIVE,
                    true,
                    false,
                    WorldType.field_77137_b
                ),
                "fake"
            ),
            new WorldProvider() {
                @Override
                public DimensionType func_186058_p() {
                    return DimensionType.OVERWORLD;
                }
            },
            new Profiler(),
            true
        );
        field_73020_y = new FakeChunkProvider(this);
    }

    public void clear() {
        ((FakeChunkProvider) field_73020_y).chunks.clear();
    }

    @SuppressWarnings("WeakerAccess")
    public void uploadSnapshot(Snapshot snapshot) {
        for (int z = 0; z < snapshot.size.func_177952_p(); z++) {
            for (int y = 0; y < snapshot.size.func_177956_o(); y++) {
                for (int x = 0; x < snapshot.size.func_177958_n(); x++) {
                    BlockPos pos = new BlockPos(x, y, z).func_177971_a(BLUEPRINT_OFFSET);
                    if (snapshot instanceof Blueprint) {
                        ISchematicBlock schematicBlock = ((Blueprint) snapshot).palette
                            .get(((Blueprint) snapshot).data[snapshot.posToIndex(x, y, z)]);
                        if (!schematicBlock.isAir()) {
                            schematicBlock.buildWithoutChecks(this, pos);
                        }
                    }
                    if (snapshot instanceof Template) {
                        if (((Template) snapshot).data.get(snapshot.posToIndex(x, y, z))) {
                            func_175656_a(pos, Blocks.field_150371_ca.func_176223_P());
                        }
                    }
                }
            }
        }
        if (snapshot instanceof Blueprint) {
            ((Blueprint) snapshot).entities.forEach(schematicEntity ->
                schematicEntity.buildWithoutChecks(this, FakeWorld.BLUEPRINT_OFFSET)
            );
        }
    }

    @Override
    public BlockPos func_175694_M() {
        return BLUEPRINT_OFFSET;
    }

    @Override
    protected IChunkProvider func_72970_h() {
        return field_73020_y;
    }

    @Override
    protected boolean func_175680_a(int x, int z, boolean allowEmpty) {
        return true;
    }

    @Override
    public Biome func_180494_b(BlockPos pos) {
        return BIOME;
    }

    @Override
    public Biome getBiomeForCoordsBody(BlockPos pos) {
        return BIOME;
    }

    @Override
    public BiomeProvider func_72959_q() {
        return new BiomeProvider(field_72986_A) {
            @Override
            public List<Biome> func_76932_a() {
                return Collections.emptyList();
            }

            @Override
            public Biome func_180631_a(BlockPos pos) {
                return BIOME;
            }

            @Override
            public Biome func_180300_a(BlockPos pos, Biome defaultBiome) {
                return BIOME;
            }

            @Override
            public Biome[] func_76937_a(Biome[] biomes, int x, int z, int width, int height) {
                return biomes;
            }

            @Override
            public Biome[] func_76933_b(@Nullable Biome[] oldBiomeList, int x, int z, int width, int depth) {
                return oldBiomeList;
            }

            @Override
            public Biome[] func_76931_a(@Nullable Biome[] listToReuse, int x, int z, int width, int length, boolean cacheFlag) {
                return listToReuse;
            }

            @Override
            public boolean func_76940_a(int x, int z, int radius, List<Biome> allowed) {
                return false;
            }

            @Nullable
            @Override
            public BlockPos func_180630_a(int x, int z, int range, List<Biome> biomes, Random random) {
                return BlockPos.field_177992_a;
            }

            @Override
            public void func_76938_b() {
            }

            @Override
            public GenLayer[] getModdedBiomeGenerators(WorldType worldType, long seed, GenLayer[] original) {
                return original;
            }

            @Override
            public boolean func_190944_c() {
                return true;
            }

            @Override
            public Biome func_190943_d() {
                return BIOME;
            }
        };
    }
}
